// This script is used to set constants which tell us the names of the mobs
// We will spawn
// If the name is "group" than it actually spawns a group, otherwise it spawns the mob

// Randomly gets one of the six different blight groups
// possible under this spawnset
int GetBlightSpawnGroupFromSet(int nSet);

// Returns a random blight placeable. Every area has the same blight set
string GetPlaceableResRefFromBlightSet();

// Returns the res ref of a rare spawn determined by nNumber.
// The res ref will be raremob_xx where xx is the rare mob number
string GetRareSpawnResRef(int nRareNumber);

// Returns the number of a rare spawn for this area.
// The number is determine randomnly from a series of six in a group
// The group chosen is nSpawnSet, which should be the area's current spawn se
int GetRareSpawn(int nSpawnSet);

// Gets whether the mob is a rarespawn. Used in the on death script.
int GetIsRareSpawn(object oMob);

// Sets that the mob concerned is a rare spawn. This means that on death, it will
// automatically change the rarespawnstatus of the area it spawned in to inactive
void SetIsRareSpawn(object oMob);

// This is the number of the rare spawn mob, 1 - 56. This uses the database every time
int GetRareSpawnTypeFromDatabase(object oArea);

// This is the number of the rare spawn mob, 1 - 56. This uses the database every time
void SaveRareSpawnTypeToDatabase(object oArea, int nRarespawn);

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
int GetRareSpawnStatusFromDatabase(object oArea);

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
void SaveRareSpawnStatusToDatabase(object oArea, int nRarespawn);

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
int GetRareSpawnStatus(object oArea);

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
void SetRareSpawnStatus(object oArea, int nRarespawn);

// If SetGroupDetails has given us a GroupResRef that has the word
// group in front, then this fires instead. It creates a custom group
// Eg one mage, one cleric, one fighter etc
// Sets local strings on the area telling it which mobs to create
// This is used when you have 5 or 6 mobs which are different but go together
void SetCustomGroup(int nGroup, object oBox);

// It will set local ints on the spawnbox
// GroupResRef, MinMobs, MaxMobs
// This is used to spawn mobs with the res ref GroupResRef
// A random number of them, between MinMobs and MaxMobs
// However if GroupResRef starts off as group, it will
// Spawn a custom group instead. Correct usage is
// group_001, group_002 etc. MUST have three numerals
// at the end for this to work
void SetGroupDetails(int nGroup, object oBox);

// Randomly gets one of the six different groups
// possible under this spawnset
int GetSpawnGroupFromSet(int nSet);

// Randomly gets one of the ten different res refs (ie placeables)
// possible under this placeable set
string GetPlaceableResRefFromSet(int nSet);

// Gets the current spawnset this area has
int GetSpawnSet(object oArea);

// Gets the current placeable this area has
int GetPlaceableSet(object oArea);

// Sets the current spawnset this area has using nSet
void SetSpawnSet(object oArea, int nSet);

// Sets the current placeable this area has using nSet
void SetPlaceableSet(object oArea, int nSet);

// Sets the current spawnset this area has
// The spawnset is chosen randomly from six possible sets every area has
// Each spawnset in turn contains six groups. The area will spawn
// mobs randomly from the six possible groups in whatever spawn set it currently has
// nAreaNumber is the will save of the spawn box in the area
void SetNewSpawnSet(object oArea, int nAreaNumber);

// Sets the current spawnset this area has
// The spawnset is chosen randomly from six possible sets every area has
// Each spawnset in turn contains six groups. The area will spawn
// mobs randomly from the six possible groups in whatever spawn set it currently has
// nPlaceableSetNumber is the fortitude save of the spawn box in the area
void SetNewPlaceableSet(object oArea, int nPlaceableSetNumber);

// Saves this area's spawn set to the database. Use this only when a new set has been created
void SaveSpawnSetToDatabase(object oArea, int nSet);

// Gets this area's current spawn set from the databse
int GetSpawnSetFromDatabase(object oArea);

// Saves this area's spawn set to the database. Use this only when a new set has been created
void SavePlaceableSetToDatabase(object oArea, int nSet);

// Gets this area's current spawn set from the databse
int GetPlaceableSetFromDatabase(object oArea);

// Reset entire databse
void DestroyModDatabase();

int GetSpawnSet(object oArea)
{
int nReturn=GetLocalInt(oArea,"currentspawnset");
return nReturn;
}

int GetPlaceableSet(object oArea)
{
int nReturn=GetLocalInt(oArea,"currentplaceableset");
return nReturn;
}



void SetCustomGroup(int nGroup, object oBox)
{
// MUST set group number every time

switch (nGroup)
{
 case 1:
 SetLocalInt(oBox, "group_number", 4);
 SetLocalString(oBox, "group_1", "NW_SKELETON");
 SetLocalString(oBox, "group_2", "NW_GHOUL");
 SetLocalString(oBox, "group_3", "NW_MUMMY");
 SetLocalString(oBox, "group_4", "NW_WRAITH");
 break;
 case 2:
 SetLocalInt(oBox, "group_number", 4);
 SetLocalString(oBox, "group_1", "NW_ORCA");
 SetLocalString(oBox, "group_2", "NW_ORCB");
 SetLocalString(oBox, "group_3", "NW_OrcChiefA");
 SetLocalString(oBox, "group_4", "NW_ORCWIZB");
 break;
 }
}


// It will set local ints on the spawn box
// GroupResRef, MinMobs, MaxMobs
// Make GroupResRef group_001 or 002 etc
// For it to be a custom group
// MUST have three numerals at the end
void SetGroupDetails(int nGroup, object oBox)
{
 switch (nGroup)
 {
 case 1:
 SetLocalString(oBox, "GroupResRef", "NW_SKELETON");
 SetLocalInt(oBox, "MaxMobs", 4);
 SetLocalInt(oBox, "MinMobs", 2);
 break;
 case 2:
 SetLocalString(oBox, "GroupResRef", "NW_MUMMY");
 SetLocalInt(oBox, "MaxMobs", 4);
 SetLocalInt(oBox, "MinMobs", 2);
 break;
 case 3:
 SetLocalString(oBox, "GroupResRef", "group_001");
 SetLocalInt(oBox, "MaxMobs", 4);
 SetLocalInt(oBox, "MinMobs", 2);
 break;
 case 4:
 SetLocalString(oBox, "GroupResRef", "group_002");
 SetLocalInt(oBox, "MaxMobs", 4);
 SetLocalInt(oBox, "MinMobs", 2);
 break;
 case 5:
 SetLocalString(oBox, "GroupResRef", "zep_shade001");
 SetLocalInt(oBox, "MaxMobs", 4);
 SetLocalInt(oBox, "MinMobs", 2);
 break;
 }

}

string GetPlaceableResRefFromSet(int nSet)
{
 string sResRef;
 int nRandom=d10();
 if (nSet==1)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_cedar_tree"; break;
  case 2: sResRef = "jw_cedar_tree"; break;
  case 3: sResRef = "jw_cedar_tree"; break;
  case 4: sResRef = "jw_cedar_tree"; break;
  case 5: sResRef = "jw_cedar_medium"; break;
  case 6: sResRef = "jw_cedar_medium"; break;
  case 7: sResRef = "jw_cedar_medium"; break;
  case 8: sResRef = "jw_cedar_small"; break;
  case 9: sResRef = "jw_cedar_small"; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==2)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_cherry_tree"; break;
  case 2: sResRef = "jw_cherry_tree"; break;
  case 3: sResRef = "jw_cherry_tree"; break;
  case 4: sResRef = "jw_cherry_medium"; break;
  case 5: sResRef = "jw_cherry_medium"; break;
  case 6: sResRef = "jw_cherry_small"; break;
  case 7: sResRef = "jw_cherry_small"; break;
  case 8: sResRef = "jw_cherry_small"; break;
  case 9: sResRef = "jw_cherry_small"; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==3)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_weirwood_tr"; break;
  case 2: sResRef = "jw_weirwood_tr"; break;
  case 3: sResRef = "jw_weirwood_me"; break;
  case 4: sResRef = "jw_weirwood_me"; break;
  case 5: sResRef = "jw_weirwood_me"; break;
  case 6: sResRef = "jw_weirwood_sm"; break;
  case 7: sResRef = "jw_weirwood_sm"; break;
  case 8: sResRef = "jw_weirwood_sm"; break;
  case 9: sResRef = "jw_weirwood_sm"; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==4)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_oak_tree"; break;
  case 2: sResRef = "jw_oak_tree"; break;
  case 3: sResRef = "jw_oak_tree"; break;
  case 4: sResRef = "jw_oak_tree"; break;
  case 5: sResRef = "jw_oak_tree"; break;
  case 6: sResRef = "jw_oak_tree"; break;
  case 7: sResRef = "jw_oak_small"; break;
  case 8: sResRef = "jw_oak_small"; break;
  case 9: sResRef = "jw_oak_small"; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==5)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_lead_vein"; break;
  case 2: sResRef = "jw_lead_vein"; break;
  case 3: sResRef = "jw_lead_vein"; break;
  case 4: sResRef = "jw_lead_vein"; break;
  case 5: sResRef = "jw_lead_vein"; break;
  case 6: sResRef = "jw_lead_vein"; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==6)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_adam_vein"; break;
  case 2: sResRef = "jw_adam_vein"; break;
  case 3: sResRef = "jw_adam_vein"; break;
  case 4: sResRef = "jw_adam_vein"; break;
  case 5: sResRef = "jw_adam_vein"; break;
  case 6: sResRef = "jw_adam_vein"; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==7)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_mith_vein"; break;
  case 2: sResRef = "jw_mith_vein"; break;
  case 3: sResRef = "jw_mith_vein"; break;
  case 4: sResRef = "jw_mith_vein"; break;
  case 5: sResRef = ""; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==8)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_mith_vein"; break;
  case 2: sResRef = "jw_adam_vein"; break;
  case 3: sResRef = "jw_adam_vein"; break;
  case 4: sResRef = "jw_lead_vein"; break;
  case 5: sResRef = "jw_lead_vein"; break;
  case 6: sResRef = "jw_lead_vein"; break;
  case 7: sResRef = "jw_lead_vein"; break;
  case 8: sResRef = "jw_adam_vein"; break;
  case 9: sResRef = "jw_lead_vein"; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==9)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_mith_vein"; break;
  case 2: sResRef = "jw_mith_vein"; break;
  case 3: sResRef = "jw_adam_vein"; break;
  case 4: sResRef = "jw_adam_vein"; break;
  case 5: sResRef = "jw_adam_vein"; break;
  case 6: sResRef = "jw_adam_vein"; break;
  case 7: sResRef = "jw_adam_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==10)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_silver_vein"; break;
  case 2: sResRef = "jw_silver_vein"; break;
  case 3: sResRef = "jw_silver_vein"; break;
  case 4: sResRef = "jw_silver_vein"; break;
  case 5: sResRef = "jw_silver_vein"; break;
  case 6: sResRef = "jw_silver_vein"; break;
  case 7: sResRef = "jw_silver_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==11)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_gold_vein"; break;
  case 2: sResRef = "jw_gold_vein"; break;
  case 3: sResRef = "jw_gold_vein"; break;
  case 4: sResRef = "jw_gold_vein"; break;
  case 5: sResRef = "jw_gold_vein"; break;
  case 6: sResRef = "jw_gold_vein"; break;
  case 7: sResRef = "jw_gold_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==12)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_silver_vein"; break;
  case 2: sResRef = "jw_silver_vein"; break;
  case 3: sResRef = "jw_silver_vein"; break;
  case 4: sResRef = "jw_silver_vein"; break;
  case 5: sResRef = "jw_silver_vein"; break;
  case 6: sResRef = "jw_gold_vein"; break;
  case 7: sResRef = "jw_gold_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==13)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_cloth_plant"; break;
  case 2: sResRef = "jw_dryad_plant"; break;
  case 3: sResRef = "jw_weed_plant"; break;
  case 4: sResRef = "jw_hyssop_plant"; break;
  case 5: sResRef = "jw_rose_plant"; break;
  case 6: sResRef = "jw_sukake_plant"; break;
  case 7: sResRef = "jw_vervain_plant"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==14)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_cloth_plant"; break;
  case 2: sResRef = "jw_cloth_plant"; break;
  case 3: sResRef = "jw_cloth_plant"; break;
  case 4: sResRef = "jw_cloth_plant"; break;
  case 5: sResRef = "jw_cloth_plant"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==15)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_dryad_plant"; break;
  case 2: sResRef = "jw_dryad_plant"; break;
  case 3: sResRef = "jw_dryad_plant"; break;
  case 4: sResRef = "jw_dryad_plant"; break;
  case 5: sResRef = "jw_dryad_plant"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==16)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_weed_plant"; break;
  case 2: sResRef = "jw_weed_plant"; break;
  case 3: sResRef = "jw_weed_plant"; break;
  case 4: sResRef = "jw_weed_plant"; break;
  case 5: sResRef = "jw_weed_plant"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==17)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_hyssop_plant"; break;
  case 2: sResRef = "jw_hyssop_plant"; break;
  case 3: sResRef = "jw_hyssop_plant"; break;
  case 4: sResRef = "jw_hyssop_plant"; break;
  case 5: sResRef = "jw_hyssop_plant"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==18)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_rose_plant"; break;
  case 2: sResRef = "jw_rose_plant"; break;
  case 3: sResRef = "jw_rose_plant"; break;
  case 4: sResRef = "jw_rose_plant"; break;
  case 5: sResRef = "jw_rose_plant"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==19)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_sukake_plant"; break;
  case 2: sResRef = "jw_sukake_plant"; break;
  case 3: sResRef = "jw_sukake_plant"; break;
  case 4: sResRef = "jw_sukake_plant"; break;
  case 5: sResRef = "jw_sukake_plant"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==20)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_vervain_plant"; break;
  case 2: sResRef = "jw_vervain_plant"; break;
  case 3: sResRef = "jw_vervain_plant"; break;
  case 4: sResRef = "jw_vervain_plant"; break;
  case 5: sResRef = "jw_vervain_plant"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==21)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_emerald_vein"; break;
  case 2: sResRef = "jw_emerald_vein"; break;
  case 3: sResRef = "jw_emerald_vein"; break;
  case 4: sResRef = "jw_emerald_vein"; break;
  case 5: sResRef = "jw_emerald_vein"; break;
  case 6: sResRef = "jw_emerald_vein"; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==22)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_ruby_vein"; break;
  case 2: sResRef = "jw_ruby_vein"; break;
  case 3: sResRef = "jw_ruby_vein"; break;
  case 4: sResRef = "jw_ruby_vein"; break;
  case 5: sResRef = "jw_ruby_vein"; break;
  case 6: sResRef = "jw_ruby_vein"; break;
  case 7: sResRef = "jw_ruby_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==23)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_diamond_vein"; break;
  case 2: sResRef = "jw_diamond_vein"; break;
  case 3: sResRef = "jw_diamond_vein"; break;
  case 4: sResRef = "jw_diamond_vein"; break;
  case 5: sResRef = "jw_diamond_vein"; break;
  case 6: sResRef = "jw_diamond_vein"; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==24)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_ruby_vein"; break;
  case 2: sResRef = "jw_ruby_vein"; break;
  case 3: sResRef = "jw_ruby_vein"; break;
  case 4: sResRef = "jw_diamond_vein"; break;
  case 5: sResRef = "jw_diamond_vein"; break;
  case 6: sResRef = "jw_ruby_vein"; break;
  case 7: sResRef = "jw_ruby_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==25)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_topaz_vein"; break;
  case 2: sResRef = "jw_topaz_vein"; break;
  case 3: sResRef = "jw_topaz_vein"; break;
  case 4: sResRef = "jw_topaz_vein"; break;
  case 5: sResRef = "jw_topaz_vein"; break;
  case 6: sResRef = "jw_topaz_vein"; break;
  case 7: sResRef = "jw_topaz_vein"; break;
  case 8: sResRef = "jw_topaz_vein"; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==26)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_rhodo_vein"; break;
  case 2: sResRef = "jw_rhodo_vein"; break;
  case 3: sResRef = "jw_rhodo_vein"; break;
  case 4: sResRef = "jw_rhodo_vein"; break;
  case 5: sResRef = "jw_rhodo_vein"; break;
  case 6: sResRef = "jw_rhodo_vein"; break;
  case 7: sResRef = "jw_rhodo_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==27)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_alex_vein"; break;
  case 2: sResRef = "jw_alex_vein"; break;
  case 3: sResRef = "jw_alex_vein"; break;
  case 4: sResRef = "jw_alex_vein"; break;
  case 5: sResRef = "jw_alex_vein"; break;
  case 6: sResRef = "jw_alex_vein"; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==28)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_topaz_vein"; break;
  case 2: sResRef = "jw_topaz_vein"; break;
  case 3: sResRef = "jw_topaz_vein"; break;
  case 4: sResRef = "jw_topaz_vein"; break;
  case 5: sResRef = "jw_topaz_vein"; break;
  case 6: sResRef = "jw_rhodo_vein"; break;
  case 7: sResRef = "jw_rhodo_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==29)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_rhodo_vein"; break;
  case 2: sResRef = "jw_rhodo_vein"; break;
  case 3: sResRef = "jw_rhodo_vein"; break;
  case 4: sResRef = "jw_rhodo_vein"; break;
  case 5: sResRef = "jw_alex_vein"; break;
  case 6: sResRef = "jw_alex_vein"; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==30)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_topaz_vein"; break;
  case 2: sResRef = "jw_topaz_vein"; break;
  case 3: sResRef = "jw_topaz_vein"; break;
  case 4: sResRef = "jw_topaz_vein"; break;
  case 5: sResRef = "jw_rhodo_vein"; break;
  case 6: sResRef = "jw_rhodo_vein"; break;
  case 7: sResRef = "jw_topaz_vein"; break;
  case 8: sResRef = "jw_alex_vein"; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==31)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_topaz_vein"; break;
  case 2: sResRef = "jw_topaz_vein"; break;
  case 3: sResRef = "jw_topaz_vein"; break;
  case 4: sResRef = "jw_topaz_vein"; break;
  case 5: sResRef = "jw_topaz_vein"; break;
  case 6: sResRef = "jw_emerald_vein"; break;
  case 7: sResRef = "jw_emerald_vein"; break;
  case 8: sResRef = "jw_emerald_vein"; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==32)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_sand_vein"; break;
  case 2: sResRef = "jw_sand_vein"; break;
  case 3: sResRef = "jw_sand_vein"; break;
  case 4: sResRef = "jw_sand_vein"; break;
  case 5: sResRef = "jw_sand_vein"; break;
  case 6: sResRef = "jw_sand_vein"; break;
  case 7: sResRef = "jw_sand_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==33)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_sand_vein"; break;
  case 2: sResRef = "jw_sand_vein"; break;
  case 3: sResRef = "jw_sand_vein"; break;
  case 4: sResRef = "jw_sand_vein"; break;
  case 5: sResRef = "jw_lead_vein"; break;
  case 6: sResRef = "jw_lead_vein"; break;
  case 7: sResRef = "jw_lead_vein"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==34)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_bee_hive"; break;
  case 2: sResRef = "jw_bee_hive"; break;
  case 3: sResRef = "jw_bee_hive"; break;
  case 4: sResRef = "jw_bee_hive"; break;
  case 5: sResRef = ""; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==35)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_cloth_plant"; break;
  case 2: sResRef = "jw_dryad_plant"; break;
  case 3: sResRef = "jw_weed_plant"; break;
  case 4: sResRef = "jw_bee_hive"; break;
  case 5: sResRef = ""; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==36)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_bee_hive"; break;
  case 2: sResRef = ""; break;
  case 3: sResRef = ""; break;
  case 4: sResRef = "jw_hyssop_plant"; break;
  case 5: sResRef = "jw_rose_plant"; break;
  case 6: sResRef = "jw_sukake_plant"; break;
  case 7: sResRef = "jw_vervain_plant"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==37)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_thumb_bush"; break;
  case 2: sResRef = "jw_thumb_bush"; break;
  case 3: sResRef = "jw_thumb_bush"; break;
  case 4: sResRef = "jw_thumb_bush"; break;
  case 5: sResRef = "jw_thumb_bush"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==38)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_winter_bush"; break;
  case 2: sResRef = "jw_winter_bush"; break;
  case 3: sResRef = "jw_winter_bush"; break;
  case 4: sResRef = "jw_winter_bush"; break;
  case 5: sResRef = "jw_winter_bush"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==39)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_moon_fung"; break;
  case 2: sResRef = "jw_moon_fung"; break;
  case 3: sResRef = "jw_moon_fung"; break;
  case 4: sResRef = "jw_moon_fung"; break;
  case 5: sResRef = "jw_moon_fung"; break;
  case 6: sResRef = "jw_moon_fung"; break;
  case 7: sResRef = "jw_moon_fung"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==40)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_emb_fung"; break;
  case 2: sResRef = "jw_emb_fung"; break;
  case 3: sResRef = "jw_emb_fung"; break;
  case 4: sResRef = "jw_emb_fung"; break;
  case 5: sResRef = "jw_emb_fung"; break;
  case 6: sResRef = "jw_emb_fung"; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==41)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_hwandi_fung"; break;
  case 2: sResRef = "jw_hwandi_fung"; break;
  case 3: sResRef = "jw_hwandi_fung"; break;
  case 4: sResRef = "jw_hwandi_fung"; break;
  case 5: sResRef = "jw_hwandi_fung"; break;
  case 6: sResRef = ""; break;
  case 7: sResRef = ""; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
else
if (nSet==42)
 {
 switch (nRandom)
  {
  case 1: sResRef = "jw_moon_fung"; break;
  case 2: sResRef = "jw_moon_fung"; break;
  case 3: sResRef = "jw_moon_fung"; break;
  case 4: sResRef = "jw_moon_fung"; break;
  case 5: sResRef = "jw_emb_fung"; break;
  case 6: sResRef = "jw_emb_fung"; break;
  case 7: sResRef = "jw_emb_fung"; break;
  case 8: sResRef = ""; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
}
// and so on
return sResRef;
}


int GetSpawnGroupFromSet(int nSet)
{
 int nRandom=d6();
 int nGroup;
 // for each set there are six different groups
 // that can be returned. Set the groups here

 // This is for our first set. The groups
 // refer to the groups up above
 if (nSet==1)
 {
  switch (nRandom)
  {
  case 1: nGroup = 1; break;
  case 2: nGroup = 2; break;
  case 3: nGroup = 3; break;
  case 4: nGroup = 4; break;
  case 5: nGroup = 1; break;
  case 6: nGroup = 2; break;
  }
 }
 else
 if (nSet==2)
 {
  switch (nRandom)
  {
  case 1: nGroup = 1; break;
  case 2: nGroup = 2; break;
  case 3: nGroup = 3; break;
  case 4: nGroup = 4; break;
  case 5: nGroup = 5; break;
  case 6: nGroup = 6; break;
  }
 }
// And so on

return nGroup;

}


void SetNewSpawnSet(object oArea, int nAreaNumber)
{
// The area number is the will save of the box called jw_area_marker
// in the area

int nRandom=d6();
 int nSet;
 // for each area there are six different sets
 // that can be returned. Set the sets here

 // This is for our first area. The sets
 // refer to the sets up above
if (nAreaNumber==1)
 {
  switch (nRandom)
  {
  case 1: nSet = 1; break;
  case 2: nSet = 1; break;
  case 3: nSet = 1; break;
  case 4: nSet = 1; break;
  case 5: nSet = 1; break;
  case 6: nSet = 1; break;
  }
 }
 else
 if (nAreaNumber==2)
 {
  switch (nRandom)
  {
  case 1: nSet = 1; break;
  case 2: nSet = 2; break;
  case 3: nSet = 3; break;
  case 4: nSet = 4; break;
  case 5: nSet = 5; break;
  case 6: nSet = 6; break;
  }
 }
// And so on


// We end by setting a local into in the area
SetLocalInt(oArea,"currentspawnset",nSet);
// And save it in the databse
SaveSpawnSetToDatabase(oArea, nSet);
}

void SetNewPlaceableSet(object oArea, int nPlaceableSetNumber)
{
// The area number is the FORTITUDE  save of the box called jw_area_marker
// in the area

int nRandom=d6();
 int nSet;
 // for each area there are six different sets
 // that can be returned. Set the sets here

 // This is for our first area. The sets
 // refer to the sets up above
if (nPlaceableSetNumber==1)
 {
  switch (nRandom)
  {
  case 1: nSet = 13; break;
  case 2: nSet = 25; break;
  case 3: nSet = 34; break;
  case 4: nSet = 35; break;
  case 5: nSet = 36; break;
  case 6: nSet = 39; break;
  }
 }
 else
 if (nPlaceableSetNumber==2)
 {
  switch (nRandom)
  {
  case 1: nSet = 1; break;
  case 2: nSet = 2; break;
  case 3: nSet = 3; break;
  case 4: nSet = 4; break;
  case 5: nSet = 5; break;
  case 6: nSet = 6; break;
  }
 }
// And so on


// We end by setting a local into in the area
SetLocalInt(oArea,"currentplaceableset",nSet);
// And save it in the databse
SavePlaceableSetToDatabase(oArea, nSet);
}

void SetSpawnSet(object oArea, int nSet)
{
SetLocalInt(oArea,"currentspawnset",nSet);
}

void SetPlaceableSet(object oArea, int nSet)
{
SetLocalInt(oArea,"currentplaceableset",nSet);
}

void SaveSpawnSetToDatabase(object oArea, int nSet)
{
SetCampaignInt("NEW_WORLD","set"+GetTag(oArea),nSet);
}

int GetSpawnSetFromDatabase(object oArea)
{
int nSet=GetCampaignInt("NEW_WORLD","set"+GetTag(oArea));
return nSet;
}

// Saves this area's spawn set to the database. Use this only when a new set has been created
void SavePlaceableSetToDatabase(object oArea, int nSet)
{
SetCampaignInt("NEW_WORLD","placeset"+GetTag(oArea),nSet);
}

// Gets this area's current spawn set from the databse
int GetPlaceableSetFromDatabase(object oArea)
{
int nSet=GetCampaignInt("NEW_WORLD","placeset"+GetTag(oArea));
return nSet;
}

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
void SetRareSpawnStatus(object oArea, int nRarespawn)
{
SetLocalInt(oArea,"rarestatus",nRarespawn);
}

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
int GetRareSpawnStatus(object oArea)
{
int nReturn=GetLocalInt(oArea,"rarestatus");
return nReturn;
}

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
int GetRareSpawnStatusFromDatabase(object oArea)
{
int nSet=GetCampaignInt("NEW_WORLD","rarestatus"+GetTag(oArea));
return nSet;
}

// Valid rarespawn status are const int RARE_SPAWN_INACTIVE;
// RARE_SPAWN_PENDING; RARE_SPAWN_SPAWNED;
void SaveRareSpawnStatusToDatabase(object oArea, int nRarespawn)
{
SetCampaignInt("NEW_WORLD","rarestatus"+GetTag(oArea),nRarespawn);
}

// This is the number of the rare spawn mob, 1 - 56. This uses the database every time
int GetRareSpawnTypeFromDatabase(object oArea)
{
int nSet=GetCampaignInt("NEW_WORLD","rarenumber"+GetTag(oArea));
return nSet;
}

// This is the number of the rare spawn mob, 1 - 56. This uses the database every time
void SaveRareSpawnTypeToDatabase(object oArea, int nRarespawn)
{
SetCampaignInt("NEW_WORLD","rarenumber"+GetTag(oArea),nRarespawn);
}

void SetIsRareSpawn(object oMob)
{
SetLocalInt(oMob,"israrespawn",TRUE);
}

int GetIsRareSpawn(object oMob)
{
int nReturn=FALSE;
if (GetLocalInt(oMob,"israrespawn")==TRUE)
{
nReturn=TRUE;
}
return nReturn;
}

int GetRareSpawn(int nSpawnSet)
{
int nRare;
int nRandom=d6();

if (nSpawnSet==1)
 {
  switch (nRandom)
  {
  case 1: nRare = 1; break;
  case 2: nRare = 1; break;
  case 3: nRare = 1; break;
  case 4: nRare = 1; break;
  case 5: nRare = 1; break;
  case 6: nRare = 1; break;
  }
 }
 else
 if (nSpawnSet==2)
 {
  switch (nRandom)
  {
  case 1: nRare = 1; break;
  case 2: nRare = 2; break;
  case 3: nRare = 3; break;
  case 4: nRare = 4; break;
  case 5: nRare = 5; break;
  case 6: nRare = 1; break;
  }
 }
// And so on

return nRare;
}

string GetRareSpawnResRef(int nRareNumber)
{
string sRare;

sRare="raremob_"+IntToString(nRareNumber);

return sRare;
}

void DestroyModDatabase()
{
DestroyCampaignDatabase("NEW_WORLD");
}

string GetPlaceableResRefFromBlightSet()
{
string sResRef;
int nRandom=d10();

switch (nRandom)
  {
  case 1: sResRef = "jw_blight_plac5"; break;
  case 2: sResRef = "jw_blight_plac4"; break;
  case 3: sResRef = "jw_blight_plac3"; break;
  case 4: sResRef = "jw_blight_plac2"; break;
  case 5: sResRef = "jw_blight_plac6"; break;
  case 6: sResRef = "jw_blight_plac2"; break;
  case 7: sResRef = "jw_blight_plac3"; break;
  case 8: sResRef = "jw_blight_plac4"; break;
  case 9: sResRef = ""; break;
  case 10: sResRef = ""; break;
 }
return sResRef;
}

int GetBlightSpawnGroupFromSet(int nSet)
{
 int nRandom=d6();
 int nGroup;
 // for each set there are six different groups
 // that can be returned. Set the groups here

 // This is for our first set. The groups
 // refer to the groups up above
 if (nSet==1)
 {
  switch (nRandom)
  {
  case 1: nGroup = 5; break;
  case 2: nGroup = 5; break;
  case 3: nGroup = 5; break;
  case 4: nGroup = 5; break;
  case 5: nGroup = 5; break;
  case 6: nGroup = 5; break;
  }
 }
 else
 if (nSet==2)
 {
  switch (nRandom)
  {
  case 1: nGroup = 1; break;
  case 2: nGroup = 2; break;
  case 3: nGroup = 3; break;
  case 4: nGroup = 4; break;
  case 5: nGroup = 5; break;
  case 6: nGroup = 6; break;
  }
 }
// And so on

return nGroup;

}

