//:://////////////////////////////////////////////
/*
Modified From Rob Bartel's Witch Wake module which was located by Hrun
the Barbarian.   Glendale from the Bioware forums was a great help in
making this all work.

Give the "FX Maker" creature (found in custom creatures under
Special Custom 1) an FX number (listed below) for its tag.  As an example if you
wanted to see green circles, you'd give the creature a tag of 3.   Place the
creature on your map where you want the effect to show up.

That's it.  It's that easy.

Area FX Addon (by Peter Poe):
Give the "FX Maker" creature an Area FX number (listed below) for its tag,
preceded by an "a". By default, these area effects will be only decorative,
but you can change the event scripts (OnEnter,Heartbeat,OnExit) in the
EffectAreaOfEffect function below to make fire hurt, bugs bug, etc...
Example: for an AOE_PER_WALLFIRE (flames on the ground) effect, the tag would be a5.

Waypoint usage:
If you want to create you FX on a different location than this creature's, create
a waypoint with a tag of "FXWP_" + the tag of this creature. The nearest waypoint
with the same tag will be chosen.
This is expecially useful to create effects on "non-walkable" locations.
Even though Visual FX work fine everywhere, Area FX seem to be "cut" around
non-walkable locations. Play with them anyway!
Example: to create green plants on a roof, give the FX Maker a tag of "2",
put a waypoint on the roof (or wherever you want), and give it a tag of "FXWP_2".

List of effects that work with this script:

VFX_DUR_PROTECTION_EVIL_MINOR  (assumed similar but lesser to above)
VFX_DUR_PROTECTION_GOOD_MINOR  (assumed similar but lesser to above)

FX NAME                           FX Number       Description

VFX_DUR_BLUR                    = 0               shimmer on floor
VFX_DUR_DARKNESS                = 1               blob of sheer darkness
VFX_DUR_ENTANGLE                = 2               green plant on floor
VFX_DUR_FREEDOM_OF_MOVEMENT     = 3               green intricate glowy circle
VFX_DUR_GLOBE_INVULNERABILITY   = 4               sphere with wiggly texture
VFX_DUR_MIND_AFFECTING_NEGATIVE = 7               3d spherical purple and white whirly flying dots
VFX_DUR_MIND_AFFECTING_POSITIVE = 8               white and blue twirling cone
VFX_DUR_GHOSTLY_VISAGE          = 9               slight blue shimmer on floor
VFX_DUR_ETHEREAL_VISAGE         = 10              purple shimmer on floor
VFX_DUR_PROT_BARKSKIN           = 11              sound but no graphic
VFX_DUR_PROT_GREATER_STONESKIN  = 12              sound but no graphic
VFX_DUR_PROT_PREMONITION        = 13              cloud of blue and purple dots and eyeballs
VFX_DUR_PROT_SHADOW_ARMOR       = 14              sound but no graphic
VFX_DUR_PROT_STONESKIN          = 15              sound but no graphic
VFX_DUR_SANCTUARY               = 16              dancing blue dots in cylindrical formation
VFX_DUR_WEB                     = 17              3d spider web formations
VFX_BEAM_LIGHTNING              = 73              plays sound but not graphic
VFX_DUR_PARALYZE_HOLD           = 82              field of white dots and white dancing cylinders
VFX_DUR_SPELLTURNING            = 138             circular lightning effect on floor
VFX_DUR_ELEMENTAL_SHIELD        = 147             circle of fire
VFX_DUR_LIGHT_BLUE_5            = 153             faint blue light
VFX_DUR_LIGHT_BLUE_10           = 154             faint xxxx light
VFX_DUR_LIGHT_BLUE_15           = 155                         "
VFX_DUR_LIGHT_BLUE_20           = 156                         "
VFX_DUR_LIGHT_YELLOW_5          = 157                         "
VFX_DUR_LIGHT_YELLOW_10         = 158                         "
VFX_DUR_LIGHT_YELLOW_15         = 159                         "
VFX_DUR_LIGHT_YELLOW_20         = 160                         "
VFX_DUR_LIGHT_PURPLE_5          = 161                         "
VFX_DUR_LIGHT_PURPLE_10         = 162                         "
VFX_DUR_LIGHT_PURPLE_15         = 163                         "
VFX_DUR_LIGHT_PURPLE_20         = 164                         "
VFX_DUR_LIGHT_RED_5             = 165                         "
VFX_DUR_LIGHT_RED_10            = 166                         "
VFX_DUR_LIGHT_RED_15            = 167                         "
VFX_DUR_LIGHT_RED_20            = 168                         "
VFX_DUR_LIGHT_ORANGE_5          = 169                         "
VFX_DUR_LIGHT_ORANGE_10         = 170                         "
VFX_DUR_LIGHT_ORANGE_15         = 171                         "
VFX_DUR_LIGHT_ORANGE_20         = 172                         "
VFX_DUR_LIGHT_WHITE_5           = 173                         "
VFX_DUR_LIGHT_WHITE_10          = 174                         "
VFX_DUR_LIGHT_WHITE_15          = 175                         "
VFX_DUR_LIGHT_WHITE_20          = 176                         "
VFX_DUR_LIGHT_GREY_5            = 177                         "
VFX_DUR_LIGHT_GREY_10           = 178                         "
VFX_DUR_LIGHT_GREY_15           = 179                         "
VFX_DUR_LIGHT_GREY_20           = 180                         "
VFX_DUR_MIND_AFFECTING_DISABLED = 208              blue and white floaty dots
VFX_DUR_MIND_AFFECTING_DOMINATED= 209              cylindrical blue, purple, & white rings that dance
VFX_BEAM_FIRE                   = 210              plays sound but not graphic
VFX_BEAM_COLD                   = 211                         "
VFX_BEAM_HOLY                   = 212                         "
VFX_BEAM_MIND                   = 213                         "
VFX_BEAM_EVIL                   = 214                         "
VFX_BEAM_ODD                    = 215                         "
VFX_BEAM_FIRE_LASH              = 216                         "
VFX_DUR_MIND_AFFECTING_FEAR     = 218              flaming flying skulls
VFX_DUR_GLOBE_MINOR             = 220              faint sphere - slightly pink
VFX_DUR_PROTECTION_ELEMENTS     = 224              ascending gold and white dots
VFX_DUR_PROTECTION_GOOD_MINOR   = 225              ascending gold dots
VFX_DUR_PROTECTION_GOOD_MAJOR   = 226              ascending gold streaks
VFX_DUR_PROTECTION_EVIL_MINOR   = 227              ascending red dots
VFX_DUR_PROTECTION_EVIL_MAJOR   = 228              ascending red streaks
VFX_DUR_MAGICAL_SIGHT           = 229              multi-colored floaty eyeballs
VFX_DUR_WEB_MASS                = 230              big spider webs on floor
VFX_DUR_PARALYZED               = 232              red and white glow on floor
VFX_DUR_ANTI_LIGHT_10           = 248              area darker & somewhat green
VFX_DUR_MAGIC_RESISTANCE        = 249              3d rings of spherical runes
VFX_DUR_AURA_COLD               = 267              big circle of dots in various colors
VFX_DUR_AURA_FIRE               = 268                       "
VFX_DUR_AURA_POISON             = 269                       "
VFX_DUR_AURA_DISEASE            = 270                       "
VFX_DUR_AURA_ODD                = 271                       "
VFX_DUR_AURA_SILENCE            = 272                       "
VFX_DUR_AURA_DRAGON_FEAR        = 291                       "
VFX_DUR_BARD_SONG               = 277              floaty music notes & song
VFX_DUR_FLAG_RED                = 303              flag model in colors
VFX_DUR_FLAG_BLUE               = 304                       "
VFX_DUR_FLAG_GOLD               = 305                       "
VFX_DUR_FLAG_PURPLE             = 306                       "
VFX_DUR_TENTACLE                = 346              dark thrashing tentacles

Area FX (note that a10 doesn't exist):
Constant:                       Tag:    Description:
AOE_PER_FOGACID                  a0     bright green fog
AOE_PER_FOGFIRE                  a1     fire fog
AOE_PER_FOGSTINK                 a2     dark green fog
AOE_PER_FOGKILL                  a3     red fog
AOE_PER_FOGMIND                  a4     blue fog
AOE_PER_WALLFIRE                 a5     flames from ground
AOE_PER_WALLWIND                 a6     no visible effect
AOE_PER_WALLBLADE                a7     spikes from ground
AOE_PER_WEB                      a8     webs
AOE_PER_ENTANGLE                 a9     green tentacles
AOE_PER_DARKNESS                 a11    complete darkness
AOE_MOB_CIRCEVIL                 a12    small circle of pink dots
AOE_MOB_CIRCGOOD                 a13    small circle of yellow dots
AOE_MOB_CIRCLAW                  a14    no visible effect
AOE_MOB_CIRCCHAOS                a15    no visible effect
AOE_MOB_FEAR                     a16    small circle of green dots
AOE_MOB_BLINDING                 a17    small circle of pink dots
AOE_MOB_UNEARTHLY                a18    small circle of green dots
AOE_MOB_MENACE                   a19    small circle of pink dots
AOE_MOB_UNNATURAL                a20    small circle of green dots
AOE_MOB_STUN                     a21    small circle of white dots
AOE_MOB_PROTECTION               a22    small circle of yellow dots
AOE_MOB_FIRE                     a23    small circle of orange dots
AOE_MOB_FROST                    a24    small circle light blue dots
AOE_MOB_ELECTRICAL               a25    small circle light blue dots
AOE_PER_FOGGHOUL                 a26    small area green fog
AOE_MOB_TYRANT_FOG               a27    medium area green fog
AOE_PER_STORM                    a28    green rain
AOE_PER_INVIS_SPHERE             a29    no visible effect
AOE_MOB_SILENCE                  a30    small circle of white dots
AOE_PER_DELAY_BLAST_FIREBALL     a31    yellow sparkles
AOE_PER_GREASE                   a32    black mist
AOE_PER_CREEPING_DOOM            a33    lots of bugs
AOE_PER_EVARDS_BLACK_TENTACLES   a34    black tentacles
AOE_MOB_INVISIBILITY_PURGE       a35    no visible effect
AOE_MOB_DRAGON_FEAR              a36    big circle of green dots
*/

void main()
{
string sTag = GetTag(OBJECT_SELF);
int nEffect;
object oFXTarget;
effect eVisualFX;
location lFXLocation;
object oArea=GetArea(OBJECT_SELF);


if (GetIsObjectValid(GetNearestObjectByTag("FXWP_" + sTag,OBJECT_SELF,1)) == TRUE) //Checks if FX has to be applied elsewhere (at the nearest waypoint with FXWP_*** tag)
    lFXLocation = GetLocation(GetNearestObjectByTag("FXWP_" + sTag,OBJECT_SELF,1));
else
    lFXLocation = GetLocation(OBJECT_SELF);
if (GetStringLeft(sTag, 1) == "a") { //Is this an Area FX...
    if (GetStringLength(sTag) == 2)
        nEffect = StringToInt(GetStringRight(sTag,1));
    else
        nEffect = StringToInt(GetStringRight(sTag,2));
    //Remove "null","null","null" if you want the specified effect's default OnEnter, Heartbeat and OnExit scripts to run.
    eVisualFX = EffectAreaOfEffect(nEffect,"null","null","null");
    AssignCommand(oArea,ApplyEffectAtLocation(DURATION_TYPE_PERMANENT, eVisualFX,lFXLocation));
    }
else { // or a Visual FX?
    nEffect = StringToInt(sTag);
    eVisualFX = EffectVisualEffect(nEffect);
    oFXTarget = CreateObject(OBJECT_TYPE_PLACEABLE, "fxbox", lFXLocation);
    AssignCommand(oArea,ApplyEffectToObject(DURATION_TYPE_PERMANENT, eVisualFX,oFXTarget));
    }
DelayCommand(1.0, DestroyObject(OBJECT_SELF));  // leave this here if you want the npc/creature destroyed
}

